####### Bookmarking useful references: #######
### Reordering factors within ggplot:
# https://sebastiansauer.github.io/ordering-bars/
###
####### Load packages #######
library(tidyverse)
library(phyloseq)
library(lme4)
library(lmerTest)
library(genefilter)
#library(doParallel)
#library("BiocParallel")
library(vegan)
library(DESeq2)
library(car)
#library(reshape2)
library(emmeans)
#library(ggpubr)
#library(RColorBrewer)
library(viridis)
library(Biostrings)
#library(multcomp)
library(multcompView)
#library(dabestr)
#library(ggdendro)

####### Define custom default ggplot theme: #######
theme_mrw <- theme_classic()+
  theme(axis.title=element_text(size=15,face='bold'),axis.text=element_text(size=13),
        legend.title=element_blank(),legend.text=element_text(size=13),
        strip.text=element_text(size=15,face='bold'),
        strip.background=element_rect(fill='grey70'))

####### Specify which functions to use: #######
# dplyr::select() clashes with MASS:select()
select <- dplyr::select
# useful function from plyr package:
mapvalues <- plyr::mapvalues

####### Define zero-tolerant function for calculating geometric means (credit: P.J. McMurdie) #######
gm_mean = function(x, na.rm=TRUE){
  exp(sum(log(x[x > 0]), na.rm=na.rm) / length(x))
}
####### Set up parallel processing: #######
nCores <- detectCores() # 8 cores available
register(MulticoreParam(nCores)) # split over 8 cores

####### Define palettes #######
taxPalette<-c("black","aquamarine2", 'chocolate1',
              'chartreuse2',"cornflowerblue","blueviolet",'aquamarine4',"darkgoldenrod1","brown1",
              "khaki1","forest green",'deeppink','khaki4',
              "lightpink","plum",'lightcoral',
              "cadetblue","palevioletred","gray50",'grey80',
              "sienna3",'thistle2','grey70','slateblue2','turquoise2','wheat3','yellow','tan1',"blue")
genoPalette <- c('red',viridis(6),viridis(6)[1],viridis(6)[2],
                 'orange',viridis(6)[1:2],viridis(6)[4:6])
matPalette <- c('cornflowerblue','limegreen','darkgoldenrod1','chocolate1')
hybPalette <- c('aquamarine3','darkorchid')

####### Function to apply variance-stabilizing transformation #######
fum.vst <- function(phyloseq_obj) {
  phyloseq_obj.vst <- phyloseq_obj %>% prune_taxa(taxa_sums(.)>0,.) # copy phyloseq object after removing any ASVs with 0 abundance
  dds <- phyloseq_to_deseq2(phyloseq_obj, ~Type+Treatment) # translate phyloseq object to DESeq2 object and specify formula
  geoMeans.hyb <- apply(counts(dds), 1, gm_mean) # apply zero-tolerant geometric mean
  dds <- estimateSizeFactors(dds, geoMeans = geoMeans.hyb) %>% varianceStabilizingTransformation(blind=FALSE,fitType = 'local') # Apply variance-stabilizing transformation
  otu_table(phyloseq_obj.vst) <- as(assay(dds),'matrix') %>% otu_table(.,taxa_are_rows = TRUE)  # Add transformed OTU table back into phyloseq object
  phyloseq_obj.vst <- transformSampleCounts(phyloseq_obj.vst,function(x) ifelse(x<0,0,x)) # Round negative values up to zeroes, to enable Bray-Curtis calculations
  rm(phyloseq_obj,dds,geoMeans.hyb) # clean up
  return(phyloseq_obj.vst)
}


####### Function to extract Bray-Curtis dissimilarity for 2 samples #######
getdist <- function(distobj, sampleA, sampleB) {
  # distobj = object of class "dist"
  # sampleA & sampleB = names of two samples
  indexA <- match(sampleA,labels(distobj)) # find the position of each sample in the distobj vector
  indexB <- match(sampleB,labels(distobj)) 
  n <- length(labels(distobj)) # get total number of samples
  distAB <- distobj[n*(indexA-1)-indexA*(indexA-1)/2+indexB-indexA]
  return(distAB)
}


####### Function: r2.LMM() => estimate R^2 for a model as correlation between fitted and observed values #######
r2.LMM <- function(m) {
  lmfit <-  lm(model.response(model.frame(m)) ~ fitted(m))
  summary(lmfit)$r.squared
}
####### Function: pve.LMM() => estimate percent variance explained by one term ######
pve.LMM <- function(lmm,term) {
  pve <- r2.LMM(lmm) - r2.LMM(update(lmm,as.formula(paste0('.~.-',term)))) # difference in r-squared with & without term
  pve <- round(100*pve,2) # multiply by 100 and round to 2 decimal plaes to make it a %
  return(pve)
}

####### Accessory functions for correlation matrices #######
# Source: http://www.sthda.com/english/wiki/ggplot2-quick-correlation-matrix-heatmap-r-software-and-data-visualization

# Get lower triangle of the correlation matrix
get_lower_tri<-function(cormat){
  cormat[upper.tri(cormat)] <- NA
  return(cormat)
}
# Get upper triangle of the correlation matrix
get_upper_tri <- function(cormat){
  cormat[lower.tri(cormat)]<- NA
  return(cormat)
}
reorder_cormat <- function(cormat){
  # Use correlation between variables as distance
  dd <- as.dist((1-cormat)/2)
  hc <- hclust(dd)
  cormat <-cormat[hc$order, hc$order]
}

####### Function: stats.LMM() => extract LMM stats #######
stats.LMM <- function(lmm,response) {
  # Get and reformat fixed effects:
  stats.fixed <- rownames_to_column(as.data.frame(anova(lmm)),var='Term') %>%
    dplyr::rename(P.orig=`Pr(>F)`,test.stat=`F value`) %>%
    mutate(stat.type='F') %>% select(Term,P.orig,test.stat,stat.type,NumDF,DenDF)
  # Get and reformat random effects:
  stats.random <- rownames_to_column(as.data.frame(ranova(lmm)),var='Term') %>%
    filter(!is.na(Df)) %>%
    dplyr::rename(P.orig=`Pr(>Chisq)`,test.stat=LRT,NumDF=Df) %>%
    mutate(stat.type='ChiSq',DenDF=Inf) %>% select(Term,P.orig,test.stat,stat.type,NumDF,DenDF)
  # Estimate percent variance explained (PVE) for each term:
  pve.all <- data.frame('Term'=c(levels(factor(stats.fixed$Term)),levels(factor(stats.random$Term))))
  pve.all$PVE <- sapply(pve.all$Term,pve.LMM,lmm=lmm)
  # Merge PVE with other stats and export
  rbind(stats.fixed,stats.random) %>% 
    merge(.,pve.all) %>%
    mutate(Response=response, test.stat=signif(test.stat,3)) -> stats.all
  return(stats.all)
  }
####### Function to calculate community cohesion/connectedness #######
# Modified from C. Herren, ISME J 11, 2426-2438 (2017)
"
# for testing:
bmdr <- readRDS('intermediate_data/phyloseq_b_clean.RDS')
data <- subset_samples(bmdr,Type=='bulk_soil')
"

### ~!~!~ NOTE: when done, change default iter -> 200 or more ~!~!~ ###
### ~!~!~ also, remove line that limits ntaxa to 100
cohesion <- function(data, pers.cutoff=0.10,iter=250,tax.shuffle=T,use.custom.cors = F) {
  ####### Create necessary functions #######
  #find the number of zeroes in a vector
  zero <- function(vec){
    num.zero <- length(which(vec == 0))
    return(num.zero)
  }
  
  #create function that averages only negative values in a vector
  neg.mean <- function(vector){
    neg.vals <- vector[which(vector < 0)]
    n.mean <- mean(neg.vals)
    if(length(neg.vals) == 0) n.mean <- 0
    return(n.mean)
  }
  
  #create function that averages only positive values in a vector
  pos.mean <- function(vector){
    pos.vals <- vector[which(vector > 0)]
    p.mean <- mean(pos.vals)
    if(length(pos.vals) == 0) p.mean <- 0
    return(p.mean)
  }
  ####### workflow options #######
  ## data = a Phyloseq object
  ## pers.cutoff = Persistence cutoff (min. fraction of taxon presence) for retaining taxa in the analysis
  ## iter = The number of iterations to run for each taxon. (>= 200 is recommended)
  ## tax.shuffle = whether to use taxon/column shuffle (tax.shuffle = T) or row shuffle algorithm (tax.shuffle = F)
  ## use.custom.cors = option to input your own correlation table (Note that your correlation table MUST have the same number of taxa as the abundance table. There should be no empty (all zero) taxon vectors in the abundance table. Even if you input your own correlation table, the persistence cutoff will be applied)
  ####### read and format data #######
  # Remove empty samples and blank taxon columns:
  data <- prune_taxa(taxa_sums(data)>0,data)
  data <- prune_samples(sample_sums(data)>0,data)
  # Convert OTU table to matrix called "c"
  c <- as(otu_table(data),'matrix')
  # Save total number of individuals in each sample in the original matrix. This will be 1 if data are in relative abundance, but not if matrix c is count data
  rowsums.orig <- rowSums(c)
  # Based on persistence cutoff, define a cutoff for the number of zeroes allowed in a taxon's distribution
  zero.cutoff <- ceiling(pers.cutoff * dim(c)[1])
  # Remove taxa that are below the persistence cutoff
  d <- c[ , apply(c, 2, zero) < (dim(c)[1]-zero.cutoff) ]
  # Remove any samples that no longer have any individuals, due to removing taxa
  d <- d[rowSums(d) > 0, ]
  # Create relative abundance matrix.  
  rel.d <- d / rowsums.orig
  # Optionally, check to see what proportion of the community is retained after cutting out taxa
  hist(rowSums(rel.d))
  # Create observed correlation matrix
  cor.mat.true <- cor(rel.d)
  # Create vector to hold median otu-otu correlations for initial otu
  med.tax.cors <- vector()
  ####### Generate null model #######
  # Run this loop for the null model to get expected pairwise correlations
  # Bypass null model if the option to input custom correlation matrix is TRUE
  if(use.custom.cors == F) {
    if(tax.shuffle == T) {
      for(which.taxon in 1:dim(rel.d)[2]){ # go through one FOCAL OTU at a time
        # print('CHECKPOINT 1')
        
        #create vector to hold correlations from every permutation for each single otu
        ## perm.cor.vec.mat stands for permuted correlations vector matrix
        perm.cor.vec.mat <- vector()
        
        for(i in 1:iter){ # iterate over the specified number of times
          # print('CHECKPOINT 2')
          #Create empty matrix of same dimension as rel.d
          perm.rel.d <- matrix(numeric(0), dim(rel.d)[1], dim(rel.d)[2])
          rownames(perm.rel.d) <- rownames(rel.d)
          colnames(perm.rel.d) <- colnames(rel.d)
          
          for(j in 1:dim(rel.d)[2]){ #For each otu (NON-FOCAL OTUs)
            # Replace the original taxon vector with a permuted taxon vector
            perm.rel.d[, j ] <- sample(rel.d[ ,j ]) 
          }
          # print('CHECKPOINT 3')
          
          # Do not randomize focal column (i.e., replace permuted column with the original for the FOCAL OTU only)
          perm.rel.d[, which.taxon] <- rel.d[ , which.taxon]
          
          # Calculate correlation matrix of permuted matrix
          cor.mat.null <- cor(perm.rel.d)
          
          # For each iteration, save the vector of null matrix correlations between focal taxon and other taxa
          perm.cor.vec.mat <- cbind(perm.cor.vec.mat, cor.mat.null[, which.taxon])
          # print('---CHECKPOINT 3A---')
          
        }
        # print('---CHECKPOINT 3B---') # executes after all iterations are complete for this taxon
        # Save the median correlations between the focal taxon and all other taxa  
        med.tax.cors <- cbind(med.tax.cors, apply(perm.cor.vec.mat, 1, median))
        
        # For large datasets, this can be helpful to know how long this loop will run
        if(which.taxon %% 20 == 0){print(which.taxon)}
      } # done with this taxon, move on to the next one
      # print('CHECKPOINT 4') 
      }
    if (tax.shuffle== F) {
      for(which.taxon in 1:dim(rel.d)[2]){ # for each FOCAL OTU
        
        #create vector to hold correlations from every permutation for each single otu
        ## perm.cor.vec.mat stands for permuted correlations vector matrix
        perm.cor.vec.mat <- vector()
        
        for(i in 1:iter){
          #Create duplicate matrix to shuffle abundances
          perm.rel.d <- rel.d 
          
          #For each taxon
          for(j in 1:dim(rel.d)[1]){ 
            which.replace <- which(rel.d[j, ] > 0 ) 
            # if the focal taxon is greater than zero, take it out of the replacement vector, so the focal abundance stays the same
            which.replace.nonfocal <- which.replace[!(which.replace %in% which.taxon)]
            
            #Replace the original taxon vector with a vector where the values greater than 0 have been randomly permuted 
            perm.rel.d[j, which.replace.nonfocal] <- sample(rel.d[ j, which.replace.nonfocal]) 
          }
          
          # Calculate correlation matrix of permuted matrix
          cor.mat.null <- cor(perm.rel.d)
          
          # For each iteration, save the vector of null matrix correlations between focal taxon and other taxa
          perm.cor.vec.mat <- cbind(perm.cor.vec.mat, cor.mat.null[, which.taxon])
          
        }
        # Save the median correlations between the focal taxon and all other taxa  
        med.tax.cors <- cbind(med.tax.cors, apply(perm.cor.vec.mat, 1, median))
        
        # For large datasets, this can be helpful to know how long this loop will run
        if(which.taxon %% 20 == 0){print(which.taxon)}
      }}
    # print('CHECKPOINT 5') # done with all taxa
  }
  ####### Matrices #######
  # Debugging lines:
  print('cor.mat.true dimensions:')
  print(dim(cor.mat.true))
  print('med.tax.cors dimensions:')
  print(dim(med.tax.cors))
  
  # Save observed minus expected correlations. Use custom correlations if use.custom.cors = TRUE
  obs.exp.cors.mat <- cor.mat.true - med.tax.cors
"  
  ifelse(use.custom.cors == T, {
    obs.exp.cors.mat <- custom.cor.mat.sub}, {
      obs.exp.cors.mat <- cor.mat.true - med.tax.cors
    }
  )"
  
  diag(obs.exp.cors.mat) <- 0
  ####### Produce desired vectors of connectedness and cohesion #######
  # Calculate connectedness by averaging positive and negative observed - expected correlations
  connectedness.pos <- apply(obs.exp.cors.mat, 2, pos.mean)
  connectedness.neg <- apply(obs.exp.cors.mat, 2, neg.mean)
  
  # Calculate cohesion by multiplying the relative abundance dataset by associated connectedness
  cohesion.pos <- rel.d %*% connectedness.pos
  cohesion.neg <- rel.d %*% connectedness.neg
  
  ####
  #### Combine vectors into one list and print 
  output <- list(connectedness.neg, connectedness.pos, cohesion.neg, cohesion.pos)
  names(output) <- c("Negative Connectedness", "Positive Connectedness", "Negative Cohesion", "Positive Cohesion")
  
  return(output)
}

