# fum2019_analysis_bacteria.R
# author: Maggie R. Wagner
# maggie.r.wagner@gmail.com
# If you use or modify this code for your own use,
# please cite the data package from which you downloaded it.
# Cite as:
# Wagner, Maggie R.; Clara Tang; Fernanda Salvato; Kayla M. Clouse; Alexandria Bartlett; Shannon Sermons; Mark Hoffmann; Peter Balint-Kurti; Manuel Kleiner  (2020)
# Data from: Microbe-dependent heterosis in maize (Version 1.1.0) [Data set]. 
# Zenodo. http://doi.org/10.5281/zenodo.3785570
#
####### Load source code #######
source('../fum2019_source.R')

####### Re-load clean datasets produced in fum2019_bacteria_dataPrep.R #######
fum.bact.asv <- readRDS('intermediate_data/phyloseq_b_asv.RDS')
fum.bact.asv.vst <- readRDS('intermediate_data/phyloseq_b_asv_vst.RDS')
smd.bact <- as(sample_data(fum.bact.asv),'data.frame')
####### ------- Big picture results * #######
####### --- ASV overlap: soils vs root #######
soilASVs <- subset_samples(fum.bact.asv,Type=='soil') %>% # look at soils only
  prune_taxa(taxa_sums(.)>0,.) %>% # remove any ASVs with 0 abundance
  taxa_names() # get ASV IDs for all ASVs present

rootASVs <- subset_samples(fum.bact.asv,Type=='root') %>% # look at root samples only
  prune_taxa(taxa_sums(.)>0,.) %>% # remove any ASVs with 0 abundance
  taxa_names() # get ASV IDs for all ASVs present in root

soilonly <- setdiff(soilASVs,rootASVs)  # 704 specific to soil (out of 1307)
rhizonly <- setdiff(rootASVs,soilASVs) # 117 specific to root (out of 1307)
both <- intersect(rootASVs,soilASVs) # 486 shared (out of 1307)

rm(soilonly,rhizonly,both) # clean up

####### --- Make separate datasets for soils/root- remove absent ASVs #######
asv.soil <- subset_samples(fum.bact.asv,Type=='soil') %>% prune_taxa(soilASVs,.)
asv.soil.vst <- subset_samples(fum.bact.asv.vst,Type=='soil') %>% prune_taxa(soilASVs,.)

asv.root <- subset_samples(fum.bact.asv,Type=='root') %>% prune_taxa(rootASVs,.)
asv.root.vst <- subset_samples(fum.bact.asv.vst,Type=='root') %>% prune_taxa(rootASVs,.)

smd.soil.bact <- subset(smd.bact,Type=='soil')
smd.root.bact <- subset(smd.bact,Type=='root')

####### --- Soil samples over weeks #######
# Unconstrained ordination at the ASV level using variance-stabilized abundances, partial out sequencing depth:
pcoa.date <- subset_samples(fum.bact.asv.vst,Type=='soil') %>% prune_taxa(soilASVs,.) %>% # Only look at soil samples; remove root-specific ASVs
  ordinate(.,method='CAP',distance='bray',formula=~1+Condition(logObs.z)) # Unconstrained ordination based on Bray-Curtis dissimilarity,
# Visualize ordination:
pdf(file='plots/soil_ordination_week_bact.pdf',width=5.5,height=4)
plot_ordination(asv.soil,pcoa.date,shape='Week',color='Treatment')+
  scale_color_manual(values=viridis(5))+
  geom_point(size=2.5)+
  theme_grey()+
  theme(axis.title=element_text(size=17,face='bold'),legend.title=element_blank(),
        axis.text=element_text(size=12),
        legend.text=element_text(size=14),legend.position='top',
        panel.grid.major = element_blank(), panel.grid.minor = element_blank())+
  guides(color=guide_legend(nrow=2,byrow=TRUE),shape=guide_legend(nrow=2,byrow=TRUE))+
  coord_fixed(ratio=16.7/38) # scale plot length/width according to importance of PCs
dev.off()

rm(pcoa.date) # clean up

####### --- Soil perMANOVA #######
set.seed(777); adonis(t(as(otu_table(asv.soil.vst),'matrix'))~logObs.z+Week*Treatment,data=smd.soil.bact,
                      method='bray',by='margin',parallel=8)
####### Root community genotype*treatment #######
# Ordination
# Unconstrained ordination at the ASV level using variance-stabilized abundances, partial out sequencing depth:
pcoa.gxt <- subset_samples(fum.bact.asv.vst,Type=='root') %>% prune_taxa(rootASVs,.) %>% # Only look at root samples; remove soil-specific ASVs
  ordinate(.,method='CAP',distance='bray',formula=~1+Condition(logObs.z)) # Unconstrained ordination based on Bray-Curtis dissimilarity,
# Visualize ordination:
pdf(file='plots/root_ordination_gxt_bact.pdf',width=3.7,height=4)
plot_ordination(asv.root,pcoa.gxt,shape='Genotype',color='Treatment')+
  scale_color_manual(values=viridis(5)[c(2,3,5)])+
  geom_point(size=2.5)+
  theme_grey()+
  theme(axis.title=element_text(size=17,face='bold'),legend.title=element_blank(),
        axis.text=element_text(size=12),
        legend.text=element_text(size=14),legend.position='top',
        panel.grid.major = element_blank(), panel.grid.minor = element_blank())+
  guides(color=guide_legend(nrow=3,byrow=TRUE),shape=guide_legend(nrow=3,byrow=TRUE))+
  coord_fixed(ratio=14.2/22.9) # scale plot length/width according to importance of PCs
dev.off()

# PerMANOVA
set.seed(777); adonis(t(as(otu_table(asv.root.vst),'matrix'))~logObs.z+Genotype*Treatment,data=smd.root.bact,
                      method='bray',by='margin',parallel=8)
"                   Df SumsOfSqs MeanSqs F.Model      R2 Pr(>F)    
logObs.z            1   0.43053 0.43053 20.6119 0.34489  0.001 ***
Genotype            2   0.08888 0.04444  2.1276 0.07120  0.036 *  
Treatment           2   0.20017 0.10008  4.7916 0.16035  0.001 ***
Genotype:Treatment  4   0.11099 0.02775  1.3284 0.08891  0.187    
Residuals          20   0.41775 0.02089         0.33465           
Total              29   1.24831                 1.00000           "
