####### Load source code #######
source('../fum2019_source.R')
####### Set parameters #######
ASVprefix <- 'b' # for bacterial ASVs
# ASVprefix <- 'f' # for fungal ASVs
####### Make sample metadata file #######
smd <- read.delim('../fum_dna.txt',sep='\t',header = TRUE) # load key linking samples to DNA libraries
summary(smd$Type) # shows we have 36 root samples + 60 soil samples
rownames(smd) <- smd$SampleID # make the sample IDs the row names for import into Phyloseq

####### Load ASV table and taxa info #######
ASV <- readRDS('seqtab_bacteria_nochim.RDS')
tax <- read.delim('taxa_bacteria.txt',sep='\t',header=TRUE)
####### Make Phyloseq object #######
bhyb <- phyloseq(otu_table(ASV,taxa_are_rows=FALSE),tax_table(as.matrix(tax)),sample_data(smd))
ntaxa(bhyb) # 8653 ASVs
# Save original phyloseq object:
saveRDS(bhyb,'intermediate_data/phyloseq_b_original.RDS')
####### Look for plant contamination: #######
plantASVs <- subset_taxa(bhyb,Family=='mitochondria' | str_detect(Class,'Chloroplast')) 
sum(taxa_sums(plantASVs)) # 29427 observations
sum(taxa_sums(bhyb)) # out of 4558851

####### Remove bad ASVs and record usable reads for each sample #######
# Remove ASVs unclassified at Kingdom level + mitochondria + chloroplast
bhyb.nobadASVs<-subset_taxa(bhyb,Kingdom%in%c('Bacteria','Archaea') & Family!='mitochondria' & Class!='Chloroplast')
ntaxa(bhyb.nobadASVs) # 4677 ASVs remain
sample_data(bhyb.nobadASVs)$UsableReads<-sample_sums(bhyb.nobadASVs)

####### Calculate alpha diversity before thresholding: #######
# Calculate alpha diversity and add to data frame:
smd <- estimate_richness(bhyb.nobadASVs,measures=c('Shannon','ACE','Observed')) %>%
  as.data.frame() %>% merge(.,sample_data(bhyb.nobadASVs),by='row.names') %>%
  column_to_rownames('SampleID') %>% dplyr::rename(SampleID=Row.names)

sample_data(bhyb.nobadASVs) <- smd  # update in phyloseq object

####### Across-sample thresholding: 5x25 (throw out "non-reproducible" ASVs) #######
threshold<-kOverA(3,A=25) # set threshold values (require k samples with A reads)
bhyb.nobadASVs.thresholded<-filter_taxa(bhyb.nobadASVs,threshold,TRUE)
ntaxa(bhyb.nobadASVs.thresholded) # 1307 ASVs remain
ntaxa(bhyb.nobadASVs) # 4677 ASVs originally
# What proportion of original reads remain?
sum(taxa_sums(bhyb.nobadASVs.thresholded))/(sum(taxa_sums(bhyb.nobadASVs)))
# 91.2 % of reads remain after thresholding

####### Remove samples with <500 usable reads #######
bhyb.nobadASVs.thresholded.highcoverage<-subset_samples(bhyb.nobadASVs.thresholded,UsableReads>=500) %>% prune_taxa(taxa_sums(.)>0,.)
nsamples(bhyb.nobadASVs.thresholded.highcoverage) # 75 samples remain
####### Save total number of observations in sample metadata #######
# This is the nuisance variable we will include in statistical models to control for sequencing depth # 
sample_data(bhyb.nobadASVs.thresholded.highcoverage)$Obs<-sample_sums(bhyb.nobadASVs.thresholded.highcoverage) # add up "good" ASV observations in each sample
sample_data(bhyb.nobadASVs.thresholded.highcoverage)$logObs<-log(sample_data(bhyb.nobadASVs.thresholded.highcoverage)$Obs) # store natural log of "good" ASV observations

# how many observations in main dataset?
mean(sample_data(bhyb.nobadASVs.thresholded.highcoverage)$Obs) # mean = 43109 observations
median(sample_data(bhyb.nobadASVs.thresholded.highcoverage)$Obs) # median = 30109 observations
sd(sample_data(bhyb.nobadASVs.thresholded.highcoverage)$Obs) # sd = 44203 observations

# how many ASVs?
ntaxa(bhyb.nobadASVs.thresholded.highcoverage) # 1307 bASVs

####### Extract sample data for "final" dataset #######
fum.bact <- bhyb.nobadASVs.thresholded.highcoverage # rename final phyloseq object
smd.bact <- as(sample_data(fum.bact),'data.frame')
smd.bact$SampleID <- row.names(smd.bact) # Store Sample IDs as a column
summary(smd.bact) # Make sure everything looks right

####### Update sample data with Z-transformation of sequencing depth #######
smd.bact <- mutate(smd.bact, logObs.z = (logObs-mean(logObs))/sd(logObs))
row.names(smd.bact) <- smd.bact$SampleID
# Add back into phyloseq object:
sample_data(fum.bact) <- smd.bact

####### Quick unconstrained ordination to check for outliers #######
# transform ASV counts to proportions / relative abundance and ordinate:
fum.bact.prop <- transform_sample_counts(fum.bact,function(ASV) ASV/sum(ASV))
# Ordinate:
pcoa.fum.bact.prop <- ordinate(fum.bact.prop,method='CAP',distance='bray',formula=~1+Condition(logObs.z))
# Visualize quick ordination:
plot_ordination(fum.bact.prop,pcoa.fum.bact.prop,color='Treatment',shape='Type') # no extreme outliers
# Clean up
rm(pcoa.fum.bact.prop,fum.bact.prop)

####### Relabel ASVs for convenience #######
tax <- as(tax_table(fum.bact),'matrix')
ASV <- as(otu_table(fum.bact),'matrix')

# Make sure ASV sequences match in taxonomy & ASV tables
setdiff(rownames(tax),colnames(ASV)) # no differences
setdiff(colnames(ASV),rownames(tax)) # no differences

# For security, make a named list of sequences and ASV_IDs to use in relabeling:
ASVlist <- paste0(ASVprefix,'ASV_',1:ncol(ASV)) # list of ASV_IDs
names(ASVlist) <- colnames(ASV) # name ASV_IDs by their sequences

# Tax table: add column to hold sequences, THEN relabel rows
tax <- transform(tax, Sequence = row.names(tax))
for (i in 1:nrow(tax)) {
  seq <- tax[['Sequence']][i]
  row.names(tax)[i] <- ASVlist[[seq]] # rename rows by matching ASV_IDs from named list
  print(paste0('relabeling ',ASVprefix,'ASV_',i,'...')) }

# ASV table: relabel columns
for (i in 1:ncol(ASV)) { 
  seq <- colnames(ASV)[i]
  colnames(ASV)[i] <- ASVlist[[seq]] # rename columns by matching ASV_IDs from named list
  print(paste0('relabeling ',ASVprefix,'ASV_',i,' out of ',ncol(ASV),'...'))
}

# Add relabeled tax & OTU tables back into phyloseq object:
fum.bact <- phyloseq(otu_table(ASV,taxa_are_rows=FALSE),tax_table(as(tax,'matrix')))
sample_data(fum.bact) <- smd.bact

# Save cleaned dataset to file:
saveRDS(fum.bact,'intermediate_data/phyloseq_b_asv.RDS')

####### Variance-stabilizing transformations #######
# Apply variance-stabilizing transformation (from DESeq2) - see fum2017_source.R for details on fum.vst() (it's just a wrapper for DESeq2 function)
fum.bact.asv.vst <- fum.vst(fum.bact) 

# Save transformed dataset to file:
saveRDS(fum.bact.asv.vst,file='intermediate_data/phyloseq_b_asv_vst.RDS')

####### store taxonomic table  #######
allTax <- as.data.frame(as(tax_table(fum.bact),'matrix')) %>% rownames_to_column(var='Taxon')
saveRDS(allTax,file='intermediate_data/tax_dataframe_bact.RDS')
####### Save processed data #######
saveRDS(smd.bact,'intermediate_data/smd_b_clean.RDS')
# Clean up:
rm(i,seq,ASV,bfum.nobadASVs,bfum.nobadASVs.WSfiltered,bfum.nobadASVs.WSfiltered.thresholded,bfum.nobadASVs.WSfiltered.thresholded.plants,bfum.nobadASVs.WSfiltered.thresholded.plants.highcoverage,bfum)

