# fum2019_analysis_fungi.R
# author: Maggie R. Wagner
# maggie.r.wagner@gmail.com
# If you use or modify this code for your own use,
# please cite the data package from which you downloaded it.
# Cite as:
# Wagner, Maggie R.; Clara Tang; Fernanda Salvato; Kayla M. Clouse; Alexandria Bartlett; Shannon Sermons; Mark Hoffmann; Peter Balint-Kurti; Manuel Kleiner  (2020)
# Data from: Microbe-dependent heterosis in maize (Version 1.1.0) [Data set]. 
# Zenodo. http://doi.org/10.5281/zenodo.3785570
#
####### Load source code #######
source('../fum2019_source.R')

####### Re-load clean datasets produced in fum2019_fungi_dataPrep.R #######
fum.fung.asv <- readRDS('intermediate_data/phyloseq_f_asv.RDS')
fum.fung.asv.vst <- readRDS('intermediate_data/phyloseq_f_asv_vst.RDS')
smd.fung <- as(sample_data(fum.fung.asv),'data.frame')
####### ------- Big picture results * #######

####### --- ASV overlap: soils vs root #######
soilASVs <- subset_samples(fum.fung.asv,Type=='soil') %>% # look at soils only
  prune_taxa(taxa_sums(.)>0,.) %>% # remove any ASVs with 0 abundance
  taxa_names() # get ASV IDs for all ASVs present

rootASVs <- subset_samples(fum.fung.asv,Type=='root') %>% # look at root samples only
  prune_taxa(taxa_sums(.)>0,.) %>% # remove any ASVs with 0 abundance
  taxa_names() # get ASV IDs for all ASVs present in root

soilonly <- setdiff(soilASVs,rootASVs)  # 74 specific to soil (out of 122)
rhizonly <- setdiff(rootASVs,soilASVs) # 9 specific to root (out of 122)
both <- intersect(rootASVs,soilASVs) # 39 shared (out of 122)

rm(soilonly,rhizonly,both) # clean up

####### --- Make separate datasets for soils/root- remove absent ASVs #######
asv.soil <- subset_samples(fum.fung.asv,Type=='soil') %>% prune_taxa(soilASVs,.)
asv.soil.vst <- subset_samples(fum.fung.asv.vst,Type=='soil') %>% prune_taxa(soilASVs,.)

asv.root <- subset_samples(fum.fung.asv,Type=='root') %>% prune_taxa(rootASVs,.)
asv.root.vst <- subset_samples(fum.fung.asv.vst,Type=='root') %>% prune_taxa(rootASVs,.)

smd.soil.fung <- subset(smd.fung,Type=='soil')
smd.root.fung <- subset(smd.fung,Type=='root')

####### --- Soil samples over weeks #######
# Unconstrained ordination at the ASV level using variance-stabilized abundances, partial out sequencing depth:
pcoa.date <- subset_samples(fum.fung.asv.vst,Type=='soil') %>% prune_taxa(soilASVs,.) %>% # Only look at soil samples; remove root-specific ASVs
  ordinate(.,method='CAP',distance='bray',formula=~1+Condition(logObs.z)) # Unconstrained ordination based on Bray-Curtis dissimilarity,
# Visualize ordination:
pdf(file='plots/soil_ordination_week_fung.pdf',width=5.5,height=4)
plot_ordination(asv.soil,pcoa.date,shape='Week',color='Treatment')+
  scale_color_manual(values=viridis(5))+
  geom_point(size=2.5)+
  #stat_ellipse(size=0.7)+
  theme_grey()+
  theme(axis.title=element_text(size=17,face='bold'),legend.title=element_blank(),
        axis.text=element_text(size=12),
        legend.text=element_text(size=14),legend.position='top',
        panel.grid.major = element_blank(), panel.grid.minor = element_blank())+
  guides(color=guide_legend(nrow=2,byrow=TRUE),shape=guide_legend(nrow=2,byrow=TRUE))+
  coord_fixed(ratio=14.9/26.3) # scale plot length/width according to importance of PCs
dev.off()

rm(pcoa.date) # clean up

####### --- Soil perMANOVA #######
# PerMANOVA
set.seed(777); adonis(t(as(otu_table(asv.soil.vst),'matrix'))~logObs.z+Week*Treatment,data=smd.soil.fung,
                      method='bray',by='margin',parallel=8)
""
####### Root community genotype*treatment #######
# Ordination
# Unconstrained ordination at the ASV level using variance-stabilized abundances, partial out batch/sampling effort:
pcoa.gxt <- subset_samples(fum.fung.asv.vst,Type=='root') %>% prune_taxa(rootASVs,.) %>% # Only look at root samples; remove soil-specific ASVs
  ordinate(.,method='CAP',distance='bray',formula=~1+Condition(logObs.z)) # Unconstrained ordination based on Bray-Curtis dissimilarity,
# Visualize ordination:
pdf(file='plots/root_ordination_gxt_fung.pdf',width=3.7,height=4)
plot_ordination(asv.root,pcoa.gxt,shape='Genotype',color='Treatment')+
  scale_color_manual(values=viridis(5)[c(2,3,5)])+
  geom_point(size=2.5)+
  theme_grey()+
  theme(axis.title=element_text(size=17,face='bold'),legend.title=element_blank(),
        axis.text=element_text(size=12),
        legend.text=element_text(size=14),legend.position='top',
        panel.grid.major = element_blank(), panel.grid.minor = element_blank())+
  guides(color=guide_legend(nrow=3,byrow=TRUE),shape=guide_legend(nrow=3,byrow=TRUE))+
  coord_fixed(ratio=19.2/24.3) # scale plot length/width according to importance of PCs
dev.off()

# PerMANOVA
set.seed(777); adonis(t(as(otu_table(asv.root.vst),'matrix'))~logObs.z+Genotype*Treatment,data=smd.root.fung,
                      method='bray',by='margin',parallel=8)
"                   Df SumsOfSqs  MeanSqs F.Model      R2 Pr(>F)    
logObs.z            1   0.22173 0.221732  4.6772 0.13359  0.001 ***
Genotype            2   0.27367 0.136834  2.8864 0.16489  0.001 ***
Treatment           2   0.31076 0.155379  3.2775 0.18723  0.001 ***
Genotype:Treatment  4   0.23730 0.059324  1.2514 0.14297  0.202    
Residuals          13   0.61629 0.047407         0.37132           
Total              22   1.65975                  1.00000         "