####### Load source code #######
source('../fum2019_source.R')
####### Set parameters #######
ASVprefix <- 'f' # for fungal ASVs
####### Make sample metadata file #######
smd <- read.delim('../fum_dna.txt',sep='\t',header = TRUE) # load key linking samples to DNA libraries
summary(smd$Type) # there are 36 root samples and 60 soil samples
rownames(smd) <- smd$SampleID # make the sample IDs the row names for import into Phyloseq

####### Load ASV table and taxa info #######
ASV <- readRDS('seqtab_fungi_nochim.RDS')
tax <- read.delim('taxa_fungi.txt',sep='\t',header=TRUE)
# Remove prefixes from taxonomic assignments:
tax <- mutate(tax,Kingdom=str_split(tax$Kingdom,'k__',simplify=TRUE)[,2],
              Phylum=str_remove(Phylum,'p__'),
         Class=str_remove(Class,'c__'),
         Order=str_remove(Order,'o__'),
         Family=str_remove(Family,'f__'),
         Genus=str_remove(Genus,'g__'),
         Species=str_remove(Species,'s__')) %>%
  column_to_rownames(var='Sequence')

####### Make Phyloseq object #######
ffum <- phyloseq(otu_table(ASV,taxa_are_rows=FALSE),tax_table(as.matrix(tax)),sample_data(smd))
ntaxa(ffum) # 871 ASVs
# Save original phyloseq object:
saveRDS(ffum,'intermediate_data/phyloseq_f_original.RDS')
####### Look for plant contamination: #######
plantASVs <- subset_taxa(ffum,str_detect(Kingdom,'Viridiplantae')) # 1 ASV
sum(taxa_sums(plantASVs)) # only 14 observations of plant sequence
sum(taxa_sums(ffum)) #  out of 1476621 total observations
####### Remove bad ASVs and record usable reads for each sample #######
# Remove ASVs unclassified at Kingdom level + mitochondria + chloroplast
ffum.nobadASVs<-subset_taxa(ffum,!str_detect(Kingdom,'Viridiplantae'))
ntaxa(ffum.nobadASVs) # 870 ASVs remain
sample_data(ffum.nobadASVs)$UsableReads<-sample_sums(ffum.nobadASVs) # store "usable" (non-plant) reads

####### Calculate alpha diversity before thresholding: #######
# Calculate alpha diversity and add to data frame:
smd <- estimate_richness(ffum.nobadASVs,measures=c('Shannon','ACE','Observed')) %>%
  as.data.frame() %>% merge(.,sample_data(ffum.nobadASVs),by='row.names') %>%
  column_to_rownames('SampleID') %>% dplyr::rename(SampleID=Row.names)

sample_data(ffum.nobadASVs) <- smd  # update in phyloseq object

####### Across-sample thresholding: 5x25 (throw out "non-reproducible" ASVs) #######
threshold<-kOverA(3,A=10) # set threshold values (require k samples with A reads)
ffum.nobadASVs.thresholded<-filter_taxa(ffum.nobadASVs,threshold,TRUE)
ntaxa(ffum.nobadASVs.thresholded) # 122 ASVs remain
ntaxa(ffum.nobadASVs) # 870 ASVs originally
# What proportion of original reads remain?
sum(taxa_sums(ffum.nobadASVs.thresholded))/(sum(taxa_sums(ffum.nobadASVs)))
# 86.0 % of reads remain after thresholding

####### Remove samples with <500 usable reads #######
ffum.nobadASVs.thresholded.highcoverage<-subset_samples(ffum.nobadASVs.thresholded,UsableReads>=500) %>% prune_taxa(taxa_sums(.)>0,.)
nsamples(ffum.nobadASVs.thresholded.highcoverage) # 57 samples remain
####### Save total number of observations in sample metadata #######
# This is the nuisance variable we will include in statistical models to control for sequencing depth # 
sample_data(ffum.nobadASVs.thresholded.highcoverage)$Obs<-sample_sums(ffum.nobadASVs.thresholded.highcoverage) # add up "good" ASV observations in each sample
sample_data(ffum.nobadASVs.thresholded.highcoverage)$logObs<-log(sample_data(ffum.nobadASVs.thresholded.highcoverage)$Obs) # store natural log of "good" ASV observations

# how many observations in main dataset?
mean(sample_data(ffum.nobadASVs.thresholded.highcoverage)$Obs) # mean = 22256 observations
median(sample_data(ffum.nobadASVs.thresholded.highcoverage)$Obs) # median = 6466 observations
sd(sample_data(ffum.nobadASVs.thresholded.highcoverage)$Obs) # sd = 29715 observations

# how many ASVs?
ntaxa(ffum.nobadASVs.thresholded.highcoverage) # 122 fASVs

####### Extract sample data for "final" dataset #######
fum.fung <- ffum.nobadASVs.thresholded.highcoverage # rename final phyloseq object
smd.fung <- as(sample_data(fum.fung),'data.frame')
smd.fung$SampleID <- row.names(smd.fung) # Store Sample IDs as a column
summary(smd.fung) # Make sure everything looks right

####### Update sample data with Z-transformation of sequencing depth #######
smd.fung <- mutate(smd.fung, logObs.z = (logObs-mean(logObs))/sd(logObs))
row.names(smd.fung) <- smd.fung$SampleID
# Add back into phyloseq object:
sample_data(fum.fung) <- smd.fung

####### Quick unconstrained ordination to check for outliers #######
# transform ASV counts to proportions / relative abundance and ordinate:
fum.fung.prop <- transform_sample_counts(fum.fung,function(ASV) ASV/sum(ASV))
# Ordinate:
pcoa.fum.fung.prop <- ordinate(fum.fung.prop,method='CAP',distance='bray',formula=~1+Condition(logObs.z))
# Visualize quick ordination:
plot_ordination(fum.fung.prop,pcoa.fum.fung.prop,color='Treatment',shape='Type') # no extreme outliers
# Clean up
rm(pcoa.fum.fung.prop,fum.fung.prop)

####### Relabel ASVs for convenience #######
tax <- as(tax_table(fum.fung),'matrix')
ASV <- as(otu_table(fum.fung),'matrix')

# Make sure ASV sequences match in taxonomy & ASV tables
setdiff(rownames(tax),colnames(ASV)) # no differences
setdiff(colnames(ASV),rownames(tax)) # no differences

# For security, make a named list of sequences and ASV_IDs to use in relabeling:
ASVlist <- paste0(ASVprefix,'ASV_',1:ncol(ASV)) # list of ASV_IDs
names(ASVlist) <- colnames(ASV) # name ASV_IDs by their sequences

# Tax table: add column to hold sequences, THEN relabel rows
tax <- transform(tax, Sequence = row.names(tax))
for (i in 1:nrow(tax)) {
  seq <- tax[['Sequence']][i]
  row.names(tax)[i] <- ASVlist[[seq]] # rename rows by matching ASV_IDs from named list
  print(paste0('relabeling ',ASVprefix,'ASV_',i,'...')) }

# ASV table: relabel columns
for (i in 1:ncol(ASV)) { 
  seq <- colnames(ASV)[i]
  colnames(ASV)[i] <- ASVlist[[seq]] # rename columns by matching ASV_IDs from named list
  print(paste0('relabeling ',ASVprefix,'ASV_',i,' out of ',ncol(ASV),'...'))
}

# Add relabeled tax & OTU tables back into phyloseq object:
fum.fung <- phyloseq(otu_table(ASV,taxa_are_rows=FALSE),tax_table(as(tax,'matrix')))
sample_data(fum.fung) <- smd.fung


# Save cleaned dataset to file:
saveRDS(fum.fung,'intermediate_data/phyloseq_f_asv.RDS')

####### Variance-stabilizing transformations #######
# Apply variance-stabilizing transformation (from DESeq2) - see fum2017_source.R for details on fum.vst() (it's just a wrapper for DESeq2 function)
fum.fung.asv.vst <- fum.vst(fum.fung) 

# Save transformed dataset to file:
saveRDS(fum.fung.asv.vst,file='intermediate_data/phyloseq_f_asv_vst.RDS')

####### store taxonomic table  #######
allTax <- as.data.frame(as(tax_table(fum.fung),'matrix')) %>% rownames_to_column(var='Taxon')
saveRDS(allTax,file='intermediate_data/tax_dataframe_fung.RDS')
####### Save processed data #######
saveRDS(smd.fung,'intermediate_data/smd_f_clean.RDS')
# Clean up:
rm(i,seq,ASV,ffum.nobadASVs,ffum.nobadASVs.WSfiltered,ffum.nobadASVs.WSfiltered.thresholded,ffum.nobadASVs.WSfiltered.thresholded.plants,ffum.nobadASVs.WSfiltered.thresholded.plants.highcoverage,ffum)
