#!/usr/bin/Rscript

# Script to run dada2 pipeline for 16S data

library(stringr)
library(ShortRead)
library(Biostrings)
library(dada2); packageVersion("dada2")

# set working directory:

wdPath <- getwd()
list.files(wdPath) # Make sure there are 'trimmed' and 'filtered' directories

# File parsing
trimpath <- paste0(wdPath,"/trimmed/") # directory containing fwd and rev trimmed reads
list.files(trimpath) # Make sure the trimmed files are in there

filtpath <- paste0(wdPath,"/filtered/") # directory containing fwd and rev filtered reads
list.files(filtpath) # Should be empty

# Load trimmed reads
fastqFs.trimmed <- sort(list.files(trimpath, pattern="trimmed.R1.fastq.gz", full.names=TRUE)) # get forward reads (search for 'R1')
fastqRs.trimmed <- sort(list.files(trimpath, pattern="trimmed.R2.fastq.gz", full.names=TRUE)) # get reverse reads (search for 'R2')
if(length(fastqFs.trimmed) != length(fastqRs.trimmed)) stop("Forward and reverse files do not match.")

# Rewrite file names for filtered reads:
fastqFs.filtered <- str_replace_all(fastqFs.trimmed,'trimmed','filtered')
fastqRs.filtered <- str_replace_all(fastqRs.trimmed,'trimmed','filtered')

# Filtering: **Adjust parameters to fit your own dataset**
# for my dataset: changing maxEE to 1 resulted in too many reads discarded
filterOutput <- filterAndTrim(fwd=fastqFs.trimmed, filt=fastqFs.filtered,
              rev=fastqRs.trimmed, filt.rev=fastqRs.filtered,
              maxEE=c(3,4), truncQ=2, maxN=0, rm.phix=TRUE, truncLen=c(220,160),
              compress=TRUE, verbose=TRUE, multithread=TRUE) 
saveRDS(filterOutput,file=paste0(wdPath,'/filterOutput.RDS'))
    
# View files that were discarded due to no reads remaining:
setdiff(fastqFs.filtered, list.files(filtpath,pattern="filtered.R1.fastq.gz", full.names=TRUE))
setdiff(fastqRs.filtered, list.files(filtpath,pattern="filtered.R2.fastq.gz", full.names=TRUE))

# Update list of filtered files to exclude discarded samples:
fastqFs.filtered <- list.files(filtpath,pattern="filtered.R1.fastq.gz", full.names=TRUE)
fastqRs.filtered <- list.files(filtpath,pattern="filtered.R2.fastq.gz", full.names=TRUE)

# Learn error rates for this run: 
errF <- learnErrors(fastqFs.filtered, nread=3e6, multithread=TRUE)
saveRDS(errF,file=paste0(wdPath,"/errF.RDS"))

errR <- learnErrors(fastqRs.filtered, nread=3e6, multithread=TRUE)
saveRDS(errR,file=paste0(wdPath,"/errR.RDS"))


# Dereplicate:
derepFs <- derepFastq(fastqFs.filtered, verbose=TRUE) 
saveRDS(derepFs,file=paste0(wdPath,"/derepFs.RDS"))
rm(derepFs) # clean up to save memory

derepRs <- derepFastq(fastqRs.filtered, verbose=TRUE)
saveRDS(derepRs,file=paste0(wdPath,"/derepRs.RDS"))
rm(derepRs) # clean up to save memory

# Name the derep-class objects by the sample names
sample.names <- str_sub(basename(fastqFs.filtered),1,7)
sample.names <- str_replace_all(sample.names,'-','_') # change hyphens to underscores if needed

# Apply core DADA2 algorithm
derepFs <- readRDS(paste0(wdPath,"/derepFs.RDS"))
names(derepFs) <- sample.names
dadaFs <- dada(derepFs, err=errF, multithread=TRUE) 
saveRDS(dadaFs,file=paste0(wdPath,"/dadaFs.RDS"))

derepRs <- readRDS(paste0(wdPath,"/derepRs.RDS"))
names(derepRs) <- sample.names
dadaRs <- dada(derepRs, err=errR, multithread=TRUE) 
saveRDS(dadaRs,file=paste0(wdPath,"/dadaRs.RDS"))

# Merge forward and reverse sequences:
mergers <- mergePairs(dadaFs, derepFs, dadaRs, derepRs, verbose=TRUE) 
saveRDS(mergers,file=paste0(wdPath,"/mergers.RDS"))

rm(derepFs)
rm(derepRs) # clean up to save memory

# Make sequence table:
seqtab <- makeSequenceTable(mergers)
saveRDS(seqtab,"seqtab_bacteria.RDS")

# Remove chimeras:
seqtab.nochim <- removeBimeraDenovo(seqtab, method="consensus", multithread=TRUE, verbose=TRUE)
saveRDS(seqtab.nochim, "seqtab_bacteria_nochim.RDS")

# Assign bacterial taxonomy (using RDP database with added plant mitochondrial sequences):
# NOTE: change path to taxonomy database as needed
taxa_bacteria <- assignTaxonomy(seqtab.nochim,"../rdp_train_set_16_plusMito.fa.gz", multithread=FALSE) 
write.table(taxa_bacteria,"taxa_bacteria.txt",sep="\t")

# Sanity check - make sure numbers of samples make sense at each step:
getN <- function(x) sum(getUniques(x))
### Because we excluded samples where all reads were filtered out, need to exclude them from the 
### original filtering step output as well to get the rows to align properly:
filterOutput2 <- filterOutput[filterOutput[,2]>0,]
track <- cbind(filterOutput2, sapply(dadaFs, getN), sapply(mergers, getN), rowSums(seqtab), rowSums(seqtab.nochim))
colnames(track) <- c("input", "filtered", "denoised", "merged", "tabled", "nonchim")
print(track)

# How many ESVs are in the final dataset?
unqs <- sort(seqtab[seqtab>0], decreasing=TRUE) %>% length
cat("DADA2 inferred", length(unqs), "sample sequences present in the community.\n")