#!/usr/bin/Rscript

library(dada2)
packageVersion("dada2")
library(ShortRead)
packageVersion("ShortRead")
library(Biostrings)
packageVersion("Biostrings")
library(stringr)
packageVersion('stringr')

# set working directory:

wdPath <- getwd()
list.files(wdPath) # Make sure there are 'trimmed' and 'filtered' directories

# File parsing
trimpath <- paste0(wdPath,"/trimmed/") # directory containing fwd and rev trimmed reads
list.files(trimpath) # Make sure the trimmed files are in there

filtpath <- paste0(wdPath,"/filtered/") # directory containing fwd and rev filtered reads
list.files(filtpath) # Should be empty

# Load trimmed reads
fastqFs.trimmed <- sort(list.files(trimpath, pattern="trimmed.R1.fastq.gz", full.names=TRUE)) # get forward reads (search for 'R1')
fastqRs.trimmed <- sort(list.files(trimpath, pattern="trimmed.R2.fastq.gz", full.names=TRUE)) # get reverse reads (search for 'R2')
if(length(fastqFs.trimmed) != length(fastqRs.trimmed)) stop("Forward and reverse files do not match.")

# Rewrite file names for filtered reads:
fastqFs.filtered <- str_replace_all(fastqFs.trimmed,'trimmed','filtered')
fastqRs.filtered <- str_replace_all(fastqRs.trimmed,'trimmed','filtered')

# Filtering: **Adjust parameters to fit your own dataset**
# for my dataset: changing maxEE to 1 resulted in too many reads discarded
filterOutput <- filterAndTrim(fwd=fastqFs.trimmed, filt=fastqFs.filtered,
              rev=fastqRs.trimmed, filt.rev=fastqRs.filtered,
              maxEE=c(2,2), truncQ=2, maxN=0, rm.phix=TRUE, minLen = 50,
              compress=TRUE, verbose=TRUE, multithread=TRUE) 
saveRDS(filterOutput,file=paste0(wdPath,'/filterOutput.RDS'))
    
# View files that were discarded due to no reads remaining:
setdiff(fastqFs.filtered, list.files(filtpath,pattern="filtered.R1.fastq.gz", full.names=TRUE))
setdiff(fastqRs.filtered, list.files(filtpath,pattern="filtered.R2.fastq.gz", full.names=TRUE))

# Update list of filtered files to exclude discarded samples:
fastqFs.filtered <- list.files(filtpath,pattern="filtered.R1.fastq.gz", full.names=TRUE)
fastqRs.filtered <- list.files(filtpath,pattern="filtered.R2.fastq.gz", full.names=TRUE)

# Learn error rates for this run: 
errF <- learnErrors(fastqFs.filtered, nread=3e6, multithread=TRUE)
saveRDS(errF,file=paste0(wdPath,"/errF.RDS"))

errR <- learnErrors(fastqRs.filtered, nread=3e6, multithread=TRUE)
saveRDS(errR,file=paste0(wdPath,"/errR.RDS"))

# Dereplicate:
derepFs <- derepFastq(fastqFs.filtered, verbose=TRUE) 
saveRDS(derepFs,file=paste0(wdPath,"/derepFs.RDS"))
rm(derepFs) # clean up to save memory

derepRs <- derepFastq(fastqRs.filtered, verbose=TRUE)
saveRDS(derepRs,file=paste0(wdPath,"/derepRs.RDS"))
rm(derepRs) # clean up to save memory

# Name the derep-class objects by the sample names
sample.names <- str_sub(basename(fastqFs.filtered),1,7)
sample.names <- str_replace_all(sample.names,'-','_') # change hyphens to underscores if needed

# Apply core DADA2 algorithm
derepFs <- readRDS(paste0(wdPath,"/derepFs.RDS"))
names(derepFs) <- sample.names
dadaFs <- dada(derepFs, err=errF, multithread=TRUE) 
saveRDS(dadaFs,file=paste0(wdPath,"/dadaFs.RDS"))

derepRs <- readRDS(paste0(wdPath,"/derepRs.RDS"))
names(derepRs) <- sample.names
dadaRs <- dada(derepRs, err=errR, multithread=TRUE) 
saveRDS(dadaRs,file=paste0(wdPath,"/dadaRs.RDS"))

# Merge forward and reverse sequences:
mergers <- mergePairs(dadaFs, derepFs, dadaRs, derepRs, verbose=TRUE) 
saveRDS(mergers,file=paste0(wdPath,"/mergers.RDS"))

rm(derepFs)
rm(derepRs) # clean up to save memory

# Make sequence table:
seqtab <- makeSequenceTable(mergers)
saveRDS(seqtab,"seqtab_fungi.RDS")

# Remove chimeras:
seqtab.nochim <- removeBimeraDenovo(seqtab, method="consensus", multithread=TRUE, verbose=TRUE)
saveRDS(seqtab.nochim, "seqtab_fungi_nochim.RDS")

# Specify location of UNITE database
# NOTE: change path to taxonomy database as needed

unite.ref <- "../sh_general_release_dynamic_s_01.12.2017_w_Zea.fasta" 

# Assign fungal taxonomy (using UNITE database):
taxa_fungi<-assignTaxonomy(seqtab.nochim,unite.ref,multithread=TRUE) 
write.table(taxa_fungi,paste0(wdPath,'/taxa_fungi.txt'),sep="\t") 

# Inspect distribution of sequence lengths:
table(nchar(getSequences(seqtab.nochim)))
 
# Sanity check / track reads through the pipeline: 
    getN <- function(x) sum(getUniques(x))
	track <- cbind(out, sapply(dadaFs, getN), sapply(dadaRs, getN), sapply(mergers, getN), rowSums(seqtab.nochim))
# If processing a single sample, remove the sapply calls: e.g. replace
# sapply(dadaFs, getN) with getN(dadaFs)
colnames(track) <- c("input", "filtered", "denoisedF", "denoisedR", "merged", "nonchim")
rownames(track) <- sample.names
head(track)