#!/bin/bash
#SBATCH
#SBATCH --array=1-96%24
#SBATCH --output=./slurmOutErr/sortTrim.%A_%a.txt
#SBATCH --error=./slurmOutErr/sortTrim.%A_%a.txt
#SBATCH --partition=short

# Adapted by M.R. Wagner from an original script written by Devin Leopold

# Script to simultaneously trim primer sequences and sort reads into 16S vs ITS subdirectories
# Optimized for high-quality 2x250 bp HiSeq output
# For longer or shorter reads, the cutadapt parameters will need to be adjusted
# Note: My primers had an extra frame-shift of 3 to 6 random nucleotides, which create variation in the trimmed sequence length

# Assumes working directory has the necessary subdirectories for storing output:
# mkdir 16S
# mkdir ITS
# mkdir 16S/pretrim
# mkdir 16S/trimmed
# mkdir ITS/pretrim
# mkdir ITS/trimmed
# mkdir slurmOutputs
# mkdir slurmErrors

# Get input files (paired forward/reverse reads)
FWD=`ls ./raw/ | grep "_R1_" | head -n $SLURM_ARRAY_TASK_ID | tail -1` # get one forward read file
REV=`echo $FWD | sed 's/_R1_/_R2_/'` # get paired reverse read

# Extract short sample identifier from file name:
# **modify this line according to your own sample name format**
SAMP=`echo $FWD | cut -b -7 | tr -d '[:space:]'`

##########################################################
##########################################################
####### SORTING + 1ST TRIMMING STEP: cutadapt   ##########
####### Separate 16S and ITS reads by detecting primers ##
####### Remove 5' gene primers from F & R reads    #######
##########################################################
##########################################################

# using 2 separate cutadapt steps to process paired reads so that I can set length limits specific to forward and reverse reads which have different primer lengths
# range of acceptable sizes are based on removing the primer + the preceding 3-6 Ns +/- 2 bp from 250-bp reads

# 16S first: trim 5` gene primer from fwd reads: search for 515f sequence
# 515f sequence = 19bp + 3-6 bp frameshift +/- 2-bp cushion = trim range of 20 to 27 bp
# Output goes into "pretrim" directory. Fwd reads have been trimmed, Rev reads have not
cutadapt -g 515f=GTGYCAGCMGCCGCGGTAA --discard-untrimmed --minimum-length 223 --maximum-length 230:300  --overlap 19 -e 0.15 --max-n=0 -o ./16S/pretrim/$SAMP.16S.pretrim.R1.fastq.gz -p ./16S/pretrim/$SAMP.16S.pretrim.R2.fastq.gz ./raw/$FWD ./raw/$REV

# 16S first: trim 5` gene primer from fwd reads: search for 806r sequence
# 806r sequence = 20 bp + 3-6 bp frameshift +/- 2-bp cushion = trim range of 21 to 28 bp
# Output goes into "trimmed" directory. Fwd and Rev reads have been trimmed
cutadapt -g 806r=GGACTACNVGGGTWTCTAAT --discard-untrimmed --minimum-length 222 --maximum-length 229:230 -p 20 -e 0.15 --max-n=0 -o ./16S/trimmed/$SAMP.16S.trimmed.R2.fastq.gz -p ./16S/trimmed/$SAMP.16S.trimmed.R1.fastq.gz ./16S/pretrim/$SAMP.16S.pretrim.R2.fastq.gz ./16S/pretrim/$SAMP.16S.pretrim.R1.fastq.gz

# ITS: trim 5` gene primer from fwd reads: search for ITS1f sequence
# ITS1f sequence = 22bp + 3-6 bp frameshift +/- 2-bp cushion = trim range of 23 to 30 bp
# Output goes into "pretrim" directory. 5' adapters have been trimmed from Fwd reads only
cutadapt -g ITS1f=CTHGGTCATTTAGAGGAASTAA --discard-untrimmed --minimum-length 220 --maximum-length 227:300 --overlap 22 -e 0.15 --max-n=0 -o ./ITS/pretrim/$SAMP.ITS.pretrim1.R1.fastq.gz -p ./ITS/pretrim/$SAMP.ITS.pretrim1.R2.fastq.gz ./raw/$FWD ./raw/$REV

# ITS: trim 5` gene primer from fwd reads: search for ITS2 sequence
# ITS2 sequence = 18 bp + 3-6 bp frameshift +/- 2-bp cushion = trim range of 19 to 26 bp
# Output goes into "pretrim" directory. 5' adapters have been trimmed from Fwd and Rev reads, but read-through (adapter contamination) not trimmed yet for either read
cutadapt -g ITS2=TTYRCTRCGTTCTTCATC --discard-untrimmed --minimum-length 224 --maximum-length 231:227 --overlap 18 -e 0.15 --max-n=0 -o ./ITS/pretrim/$SAMP.ITS.pretrim2.R2.fastq.gz -p ./ITS/pretrim/$SAMP.ITS.pretrim2.R1.fastq.gz ./ITS/pretrim/$SAMP.ITS.pretrim1.R2.fastq.gz ./ITS/pretrim/$SAMP.ITS.pretrim1.R1.fastq.gz

##########################################################
##########################################################
####### 2nd TRIMMING STEP (ITS only): SeqPurge   #########
####### Remove opposite gene primers from F & R reads  ###
##########################################################
##########################################################

# ITS: Use SeqPurge to remove read-through into opposite primers:
# -a1 = reverse complement of ITS2 (for trimming the forward read)
# -a2 = reverse complement of ITS1f (for trimming the reverse read)
# -qcut 3 -qwin 1 will remove dark cycles at end of reads (Qual=2)
# reverse complement of ITS2 = GATGAAGAACGYAGYRAA
# reverse complement of ITS1f = TTASTTCCTCTAAATGACCDAG

# Trim rcITS2 from the forward read + trim rcITS1f from the reverse read:

SeqPurge -in1 ./ITS/pretrim/$SAMP.ITS.pretrim2.R1.fastq.gz -in2 ./ITS/pretrim/$SAMP.ITS.pretrim2.R2.fastq.gz -out1 ./ITS/trimmed/$SAMP.ITS.trimmed.R1.fastq.gz -out2 ./ITS/trimmed/$SAMP.ITS.trimmed.R2.fastq.gz -a1 GATGAAGAACGYAGYRAA -a2 TTASTTCCTCTAAATGACCDAG -qcut 3 -qwin 1 -ncut 0 -min_len 100 -threads 1

##########################################################
##########################################################
####### 3rd TRIMMING STEP (ITS only): Trimmomatic   ######
####### Truncate reads to remove low-quality 3' ends  ####
##########################################################
##########################################################

# Crop reads to help with tail quality for DADA denoising 
# Truncates longer reads at a maximum length value, without discarding shorter reads
## *** NOTE: requires previewing quality of some "pretrim3" files (SeqPurge output) to decide crop values

trimmomatic SE ./ITS/pretrim/$SAMP.ITS.pretrim3.R1.fastq.gz ./ITS/trimmed/$SAMP.ITS.trimmed.R1.fastq.gz CROP:220
trimmomatic SE ./ITS/pretrim/$SAMP.ITS.pretrim3.R2.fastq.gz ./ITS/trimmed/$SAMP.ITS.trimmed.R2.fastq.gz CROP:190

# Don't forget to clean up afterwards: rm ./*/pretrim/* 
